@@ -18,6 +18,12 @@
 #' @param decompress Logicol indicating wheather need to decompress the downloaded file, default is TRUE
 #' @param dependence.need Logical should the dependence should be installed
 #' @param showWarnings Logical should the warnings on failure be shown?
+#' @param extra.list A list that can replace the configuration file '{{debug}}' by list(debug = TRUE), and {{debug}} will be setted to TRUE
+#' @param rcmd.parse Logical wheather parse '@>@str_replace('abc', 'b', 'c')@<@' in config to 'acc'
+#' @param bash.parse Logical wheather parse '#>#echo $HOME#<#' in config to your HOME PATH
+#' @param glue.parse Logical wheather parse '!!glue{1:5}' in config to ['1','2','3','4','5']; 
+#' ['nochange', '!!glue(1:5)', 'nochange'] => ['nochange', '1', '2', '3', '4', '5', 'nochange']
+#' @param glue.flag A character flage indicating wheater run glue() function to parse (Default is !!glue) 
 #' @param verbose Ligical indicating wheather show the log message
 #' @param ... Other key and value paired need be saved in BioInstaller passed to \code{\link{change.info}}
 #' @export
@@ -33,7 +39,8 @@ install.bioinfo <- function(name = c(), download.dir = c(), destdir = c(), name.
   version = c(), show.all.versions = FALSE, show.all.names = FALSE, db = Sys.getenv("BIO_SOFTWARES_DB_ACTIVE", 
     system.file("extdata", "softwares_db_demo.yaml", package = "BioInstaller")), 
   download.only = FALSE, decompress = TRUE, dependence.need = TRUE, showWarnings = FALSE, 
-  verbose = TRUE, ...) {
+  extra.list = list(), rcmd.parse = TRUE, bash.parse = TRUE, glue.parse = TRUE, 
+  glue.flag = "!!glue", verbose = TRUE, ...) {
   db.check(db)
   github.names <- eval.config.sections(file = github.cfg)
   nongithub.names <- eval.config.sections(file = nongithub.cfg)
@@ -45,8 +52,6 @@ install.bioinfo <- function(name = c(), download.dir = c(), destdir = c(), name.
   if (!show.all.versions) {
     info.msg(sprintf("Debug:name:%s", paste0(name, collapse = ", ")), verbose = verbose)
     info.msg(sprintf("Debug:destdir:%s", paste0(destdir, collapse = ", ")), verbose = verbose)
-    info.msg(sprintf("Debug:version:%s", paste0(version, collapse = ", ")), verbose = verbose)
-    info.msg(sprintf("Debug:show.all.versions:%s", show.all.versions), verbose = verbose)
     info.msg(sprintf("Debug:db:%s", db), verbose = verbose)
     info.msg(sprintf("Debug:github.cfg:%s", github.cfg), verbose = verbose)
     info.msg(sprintf("Debug:nongithub.cfg:%s", nongithub.cfg), verbose = verbose)
@@ -57,6 +62,7 @@ install.bioinfo <- function(name = c(), download.dir = c(), destdir = c(), name.
   count <- 1
   bygithub <- NULL
   bynongithub <- NULL
+  names.versions = list()
   for (i in name) {
     old_wd <- getwd()
     i <- tolower(i)
@@ -80,7 +86,8 @@ install.bioinfo <- function(name = c(), download.dir = c(), destdir = c(), name.
         github.cfg = github.cfg, name.saved = name.saved[count], version = version[count], 
         show.all.versions = show.all.versions, db = db, download.only = download.only, 
         verbose = verbose, showWarnings = showWarnings, dependence.need = dependence.need, 
-        ...)
+        extra.list = extra.list, rmcd.parse = rcmd.parse, bash.parse = bash.parse, 
+        glue.parse = glue.parse, glue.flag = glue.flag, ...)
       bygithub <- c(bygithub, i)
     } else if (i %in% nongithub.names || (sf.name %in% nongithub.names && sf.name != 
       i)) {
@@ -93,7 +100,9 @@ install.bioinfo <- function(name = c(), download.dir = c(), destdir = c(), name.
         name.saved = name.saved[count], nongithub.cfg = nongithub.cfg, version = version[count], 
         show.all.versions = show.all.versions, db = db, download.only = download.only, 
         showWarnings = showWarnings, decompress = decompress, dependence.need = dependence.need, 
-        verbose = verbose, ...)
+        verbose = verbose, extra.list = extra.list, rcmd.parse = rcmd.parse, 
+        bash.parse = bash.parse, glue.parse = glue.parse, glue.flag = glue.flag, 
+        ...)
       bynongithub <- c(bynongithub, i)
     } else {
       warning(sprintf("%s not existed in install database, so can not be installed by BioInstaller package.", 
@@ -109,17 +118,22 @@ install.bioinfo <- function(name = c(), download.dir = c(), destdir = c(), name.
       info.msg(sprintf("%s downloaded fail!", name), verbose = verbose)
       return(FALSE)
     }
-    if (show.all.versions) {
-      return(status)
-    }
     if (status == TRUE && (name.saved[count] %in% show.installed(db))) {
       install.success <- c(install.success, name.saved[count])
     } else {
       install.fail <- c(install.fail, name.saved[count])
     }
     setwd(old_wd)
+    if (show.all.versions) {
+      names.versions[[i]] <- status
+    }
     count <- count + 1
   }
+  if (show.all.versions && count == 2) {
+    return(status)
+  } else if (show.all.versions) {
+    return(names.versions)
+  }
   info.msg(sprintf("Debug:Install by Github configuration file: %s", paste0(bygithub, 
     collapse = ", ")), verbose = verbose)
   info.msg(sprintf("Debug:Install by Non Github configuration file: %s", paste0(bynongithub, 
@@ -153,6 +167,13 @@ install.bioinfo <- function(name = c(), download.dir = c(), destdir = c(), name.
 #' @param download.only Logicol indicating wheather only download source or file (non-github)
 #' @param dependence.need Logical should the dependence should be installed
 #' @param showWarnings Logical should the warnings on failure be shown?
+#' @param extra.list A list that can replace the configuration file '{{debug}}' by list(debug = TRUE), and {{debug}} will be setted to TRUE
+#' @param rcmd.parse Logical wheather parse '@>@str_replace('abc', 'b', 'c')@<@' in config to 'acc'
+#' @param bash.parse Logical wheather parse '#>#echo $HOME#<#' in config to your HOME PATH
+#' @param glue.parse Logical wheather parse '!!glue{1:5}' in config to ['1','2','3','4','5']; 
+#' ['nochange', '!!glue(1:5)', 'nochange'] => ['nochange', '1', '2', '3', '4', '5', 'nochange']
+#' @param glue.flag A character flage indicating wheater run glue() function to parse (Default is !!glue) 
+#' @param save.to.db Ligical indicating wheather save the install infomation in db
 #' @param verbose Ligical indicating wheather show the log message
 #' @param ... Other key and value paired need be saved in BioInstaller passed to \code{\link{change.info}}
 #' @return Bool Value
@@ -166,8 +187,9 @@ install.github <- function(name = "", download.dir = NULL, destdir = NULL, versi
   show.all.versions = FALSE, name.saved = NULL, github.cfg = system.file("extdata", 
     "github.toml", package = "BioInstaller"), db = Sys.getenv("BIO_SOFTWARES_DB_ACTIVE", 
     system.file("extdata", "softwares_db_demo.yaml", package = "BioInstaller")), 
-  download.only = FALSE, showWarnings = FALSE, dependence.need = TRUE, verbose = TRUE, 
-  ...) {
+  download.only = FALSE, showWarnings = FALSE, dependence.need = TRUE, extra.list = list(), 
+  rcmd.parse = TRUE, bash.parse = TRUE, glue.parse = TRUE, glue.flag = "!!glue", 
+  save.to.db = TRUE, verbose = TRUE, ...) {
   old.work.dir <- getwd()
   config.cfg <- github.cfg
   name <- tolower(name)
@@ -178,14 +200,13 @@ install.github <- function(name = "", download.dir = NULL, destdir = NULL, versi
   }
   
   config <- eval.config(config = name, file = config.cfg)
-  info.msg("Fetching Versions....", verbose = verbose)
-  all.versions <- show.avaliable.versions(config)
+  info.msg(sprintf("Fetching %s versions....", name), verbose = verbose)
+  all.versions <- show.avaliable.versions(config, name)
   if (show.all.versions) {
     return(all.versions)
   }
-  info.msg(sprintf("Avaliable versions:%s", paste0(all.versions, collapse = ", ")), 
-    verbose = verbose)
   version <- version.initial(name, version, all.versions, config)
+  info.msg(sprintf("Install versions:%s", paste0(version, collapse = ", ")), verbose = verbose)
   processed.dir.list <- pre.process.dir(name, destdir, download.dir, 1)
   destdir <- processed.dir.list[["des.dir"]]
   download.dir <- processed.dir.list[["down.dir"]]
@@ -196,18 +217,19 @@ install.github <- function(name = "", download.dir = NULL, destdir = NULL, versi
   
   args.all <- as.list(match.call())
   args.all$version <- version
-  args.all$destdir <- destdir
-  args.all$download.dir <- download.dir
+  args.all$destdir <- normalizePath(destdir, "/", FALSE)
+  args.all$download.dir <- normalizePath(download.dir, "/", FALSE)
   args.all$os.version <- get.os()
   args.all <- args.all[names(args.all) != ""]
   config <- configr::parse.extra(config = config, extra.list = args.all)
-  config <- configr::parse.extra(config = config, other.config = db)
-  config <- configr::parse.extra(config = config, rcmd.parse = T)
+  config <- configr::parse.extra(config = config, other.config = db, rcmd.parse = rcmd.parse, 
+    bash.parse = bash.parse, glue.parse = glue.parse, glue.flag = glue.flag)
   
   github_url <- config$github_url
   use_git2r <- config$use_git2r
   recursive_clone <- config$recursive_clone
-  if (is.null(github_url) || github_url == "") {
+  if ((all(is.null(github_url)) | all(github_url == "")) | (is.logical(config$no.need.download) && 
+    config$no.need.download == TRUE)) {
     need.download = FALSE
   } else {
     need.download = TRUE
@@ -237,8 +259,8 @@ install.github <- function(name = "", download.dir = NULL, destdir = NULL, versi
     process.dependence(config, db, download.dir, destdir, verbose)
     config <- eval.config(config = name, file = config.cfg)
     config <- configr::parse.extra(config = config, extra.list = args.all)
-    config <- configr::parse.extra(config, other.config = db)
-    config <- configr::parse.extra(config = config, rcmd.parse = T)
+    config <- configr::parse.extra(config = config, other.config = db, rcmd.parse = rcmd.parse, 
+      bash.parse = bash.parse, glue.parse = glue.parse, glue.flag = glue.flag)
   }
   set.makedir(make.dir, download.dir)
   before.cmd <- get.subconfig(config, "before_install")
@@ -264,9 +286,11 @@ install.github <- function(name = "", download.dir = NULL, destdir = NULL, versi
       if (!is.null(name.saved)) {
         name <- tolower(name.saved)
       }
-      change.info(name = name, installed = TRUE, source.dir = download.dir, 
-        install.dir = destdir, bin.dir = bin.dir, version = version, last.update.time = last.update.time, 
-        db = db, verbose = verbose, ...)
+      if (save.to.db) {
+        change.info(name = name, installed = TRUE, source.dir = download.dir, 
+          install.dir = destdir, bin.dir = bin.dir, version = version, last.update.time = last.update.time, 
+          db = db, verbose = verbose, ...)
+      }
     }
   } else {
     info.msg("Running after install fail steps.", verbose = verbose)
@@ -301,10 +325,17 @@ install.github <- function(name = "", download.dir = NULL, destdir = NULL, versi
 #' system.file('extdata', 'softwares_db_demo.yaml', package = 'BioInstaller'))
 #' @param download.only Logicol indicating wheather only download source or file (non-github)
 #' @param decompress Logicol indicating wheather need to decompress the downloaded file, default is TRUE
-#' @param ... Other key and value paired need be saved in BioInstaller passed to \code{\link{change.info}}
 #' @param dependence.need Logical should the dependence should be installed
 #' @param showWarnings Logical should the warnings on failure be shown?
+#' @param extra.list A list that can replace the configuration file '{{debug}}' by list(debug = TRUE), and {{debug}} will be setted to TRUE
+#' @param rcmd.parse Logical wheather parse '@>@str_replace('abc', 'b', 'c')@<@' in config to 'acc'
+#' @param bash.parse Logical wheather parse '#>#echo $HOME#<#' in config to your HOME PATH
+#' @param glue.parse Logical wheather parse '!!glue{1:5}' in config to ['1','2','3','4','5']; 
+#' ['nochange', '!!glue(1:5)', 'nochange'] => ['nochange', '1', '2', '3', '4', '5', 'nochange']
+#' @param glue.flag A character flage indicating wheater run glue() function to parse (Default is !!glue) 
+#' @param save.to.db Ligical indicating wheather save the install infomation in db
 #' @param verbose Ligical indicating wheather show the log message
+#' @param ... Other key and value paired need be saved in BioInstaller passed to \code{\link{change.info}}
 #' @return Bool Value
 #' @export
 #' @examples
@@ -317,7 +348,8 @@ install.nongithub <- function(name = "", download.dir = NULL, destdir = NULL, ve
     "nongithub.toml", package = "BioInstaller"), db = Sys.getenv("BIO_SOFTWARES_DB_ACTIVE", 
     system.file("extdata", "softwares_db_demo.yaml", package = "BioInstaller")), 
   download.only = FALSE, decompress = TRUE, dependence.need = TRUE, showWarnings = FALSE, 
-  verbose = TRUE, ...) {
+  extra.list = list(), rcmd.parse = TRUE, bash.parse = TRUE, glue.parse = TRUE, 
+  glue.flag = "!!glue", save.to.db = TRUE, verbose = TRUE, ...) {
   old.work.dir <- getwd()
   config.cfg <- nongithub.cfg
   name <- tolower(name)
@@ -328,14 +360,13 @@ install.nongithub <- function(name = "", download.dir = NULL, destdir = NULL, ve
     return(FALSE)
   }
   config <- eval.config(config = name, file = config.cfg)
-  info.msg("Fetching Versions....", verbose = verbose)
-  all.versions <- show.avaliable.versions(config)
+  info.msg(sprintf("Fetching %s versions....", name), verbose = verbose)
+  all.versions <- show.avaliable.versions(config, name)
   if (show.all.versions) {
     return(all.versions)
   }
-  info.msg(sprintf("Avaliable versions:%s", paste0(all.versions, collapse = ", ")), 
-    verbose = verbose)
   version <- version.initial(name, version, all.versions, config)
+  info.msg(sprintf("Install versions:%s", paste0(version, collapse = ", ")), verbose = verbose)
   processed.dir.list <- pre.process.dir(name, destdir, download.dir, 1)
   destdir <- processed.dir.list[["des.dir"]]
   download.dir <- processed.dir.list[["down.dir"]]
@@ -346,16 +377,18 @@ install.nongithub <- function(name = "", download.dir = NULL, destdir = NULL, ve
   
   args.all <- as.list(match.call())
   args.all$version <- version
-  args.all$destdir <- destdir
-  args.all$download.dir <- download.dir
+  args.all$destdir <- normalizePath(destdir, "/", FALSE)
+  args.all$download.dir <- normalizePath(download.dir, "/", FALSE)
   args.all$os.version <- get.os()
   args.all <- args.all[names(args.all) != ""]
+  args.all <- configr::config.list.merge(args.all, extra.list)
   config <- configr::parse.extra(config = config, extra.list = args.all)
-  config <- configr::parse.extra(config = config, other.config = db)
-  config <- configr::parse.extra(config = config, rcmd.parse = T)
+  config <- configr::parse.extra(config = config, other.config = db, rcmd.parse = rcmd.parse, 
+    bash.parse = bash.parse, glue.parse = glue.parse, glue.flag = glue.flag)
   
   source_url <- source.url.initial(config)
-  if (all(is.null(source_url)) | all(source_url == "")) {
+  if ((all(is.null(source_url)) | all(source_url == "")) | (is.logical(config$no.need.download) && 
+    config$no.need.download == TRUE)) {
     need.download <- FALSE
   } else {
     need.download <- TRUE
@@ -382,9 +415,10 @@ install.nongithub <- function(name = "", download.dir = NULL, destdir = NULL, ve
   info.msg(msg, verbose = verbose)
   info.msg("Running before install steps.", verbose = verbose)
   if (need.download && !is.download.dir(config)) {
-    extract_dir <- sprintf("%s/install_tmp/", tempdir())
-    dir.create(extract_dir, showWarnings = FALSE)
-    destfile <- sprintf("%s/%s", extract_dir, filename)
+    tmp.dir <- sprintf("%s/%s", dirname(download.dir), stri_rand_strings(1, 10))
+    dir.create(tmp.dir, recursive = TRUE)
+    tmp.dir <- normalizePath(tmp.dir, "/")
+    destfile <- sprintf("%s/%s", tmp.dir, filename)
     msg <- sprintf("Now start to download %s in %s.", name, download.dir)
     info.msg(msg, verbose = verbose)
     status <- download.dir.files(config, source_url, destfile, showWarnings, 
@@ -408,6 +442,7 @@ install.nongithub <- function(name = "", download.dir = NULL, destdir = NULL, ve
       status <- extract.file(fn, download.dir, decompress[count])
       count <- count + 1
     }
+    unlink(tmp.dir, recursive = TRUE, force = TRUE)
     if (!all(status)) {
       return(FALSE)
     }
@@ -424,16 +459,18 @@ install.nongithub <- function(name = "", download.dir = NULL, destdir = NULL, ve
     status <- config.and.name.initial(config.cfg, name)
     config <- eval.config(config = name, file = config.cfg)
     config <- configr::parse.extra(config = config, extra.list = args.all)
-    config <- configr::parse.extra(config = config, other.config = db)
-    config <- configr::parse.extra(config = config, rcmd.parse = T)
+    config <- configr::parse.extra(config = config, other.config = db, rcmd.parse = rcmd.parse, 
+      bash.parse = bash.parse, glue.parse = glue.parse, glue.flag = glue.flag)
   }
   make.dir <- config$make_dir
-  files <- list.files(download.dir)
+  all.files <- list.files(download.dir)
   set.makedir(make.dir, download.dir)
-  if ((length(files) == 1) && (length(list.files(files)) != 0)) {
-    file.copy(sprintf("%s/%s", files, list.files(files)), "./", recursive = T, 
-      copy.mode = TRUE)
-    unlink(files, recursive = TRUE, force = TRUE)
+  all.files.sub <- list.files(all.files, ".*")
+  if ((length(all.files) == 1) && (length(all.files.sub) != 0)) {
+    file.rename(sprintf("%s/%s", all.files, all.files.sub, sprintf("./", all.files.sub)))
+    if (all.files.sub == 0) {
+      unlink(all.files, recursive = TRUE, force = TRUE)
+    }
   }
   before.cmd <- get.subconfig(config, "before_install")
   status <- for_runcmd(before.cmd, verbose = verbose)
@@ -458,9 +495,11 @@ install.nongithub <- function(name = "", download.dir = NULL, destdir = NULL, ve
       if (!is.null(name.saved)) {
         name <- tolower(name.saved)
       }
-      change.info(name = name, installed = TRUE, source.dir = download.dir, 
-        install.dir = destdir, bin.dir = bin.dir, version = version, last.update.time = last.update.time, 
-        db = db, verbose = verbose, ...)
+      if (save.to.db) {
+        change.info(name = name, installed = TRUE, source.dir = download.dir, 
+          install.dir = destdir, bin.dir = bin.dir, version = version, last.update.time = last.update.time, 
+          db = db, verbose = verbose, ...)
+      }
     }
   } else {
     info.msg("Running after install fail steps.", verbose = verbose)
