@@ -48,7 +48,8 @@ check.install.name <- function(name, config.cfg) {
 # Initial parameter version
 version.initial <- function(name = "", version = NULL, versions = NULL, config = NULL) {
   if (is.null(version)) {
-    version <- version.newest(config, versions)
+    params <- list(config = config, versions = versions)
+    version <- do.call(version.newest, params)
   }
   if (is.numeric(version)) {
     version <- as.character(version)
@@ -60,12 +61,15 @@ version.initial <- function(name = "", version = NULL, versions = NULL, config =
 }
 
 # Check wheather show all avaliable version can be installed
-show.avaliable.versions <- function(config) {
+show.avaliable.versions <- function(config, name) {
   flag <- use.github.response(config)
   if (flag) {
     versions <- as.character(get.github.version(config))
   } else {
-    versions <- config$version_available
+    versions <- nongithub2versions(name)
+    if (is.null(versions)) {
+      versions <- config$version_available
+    }
   }
   return(versions)
 }
@@ -73,15 +77,19 @@ show.avaliable.versions <- function(config) {
 # Check wheather destdir is exist or not, if not will create it, and set workdir
 # in make.dir
 set.makedir <- function(make.dir, destdir) {
+  destdir <- normalizePath(destdir, "/", mustWork = FALSE)
   if (dir.exists(destdir)) {
     setwd(destdir)
   } else {
     dir.create(destdir, showWarnings = FALSE, recursive = TRUE)
     setwd(destdir)
   }
-  for (i in make.dir) {
-    if (i != "./" && dir.exists(i)) {
-      setwd(i)
+  if (!is.null(make.dir)) {
+    make.dir <- normalizePath(make.dir, "/", mustWork = FALSE)
+    for (i in make.dir) {
+      if (i != "./" && dir.exists(i)) {
+        setwd(i)
+      }
     }
   }
 }
@@ -267,7 +275,7 @@ git.download <- function(name, destdir, version, github_url, use_git2r, recursiv
   recursive_clone <- convert.bool(recursive_clone)
   if (use_git2r) {
     if (!dir.exists(dirname(destdir))) {
-      dir.create(dirname(destdir))
+      dir.create(dirname(destdir), recursive = TRUE)
     }
     repo <- git2r::clone(github_url, destdir)
     if (version != "master") {
